==> geomWatch-1.2.1.tbz: geomWatch-1.2.1/configuration.hpp <==
/*
 * Copyright 2007-2010 Boris Kochergin. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef CONFIGURATION_HPP
#define CONFIGURATION_HPP

#include <string>
#include <fstream>
#include <vector>
#include <map>
#include <limits>

#include <errno.h>
#include <string.h>

std::string getCWD() {
  char cwd[PATH_MAX];
  getcwd(cwd, PATH_MAX);
  return cwd;
}

class Configuration {
  public:
    Configuration(const std::string);
    Configuration();
    bool open(const std::string);
    operator bool() const;
    std::string error() const;
    std::string fileName() const;
    std::string getString(const std::string);
    std::vector <std::string> &getStrings(const std::string);
    size_t getNumber(const std::string);
  private:
    std::map <std::string, std::vector <std::string> > options;
    std::string _fileName;
    bool _error;
    std::string errorMessage;
};

Configuration::Configuration(const std::string fileName) {
  open(fileName);
}

Configuration::Configuration() {
}

bool Configuration::open(const std::string fileName) {
  std::string line, option, value;
  size_t delimiter, openingQuote, closingQuote;
  std::ifstream file;
  if (fileName.find('/') == std::string::npos) {
    _fileName = getCWD() + '/' + fileName;
  }
  else {
    _fileName = fileName;
  }
  _error = false;
  file.open(fileName.c_str());
  if (!file) {
    _error = true;
    errorMessage = fileName + ": " + strerror(errno);
    return false;
  }
  while (getline(file, line)) {
    delimiter = line.find('=');
    option = line.substr(0, delimiter);
    value = line.substr(delimiter + 1);
    openingQuote = value.find('"');
    if (openingQuote != std::string::npos) {
      closingQuote = value.find('"', openingQuote + 1);
      if (closingQuote != std::string::npos) {
        value = value.substr(openingQuote + 1,
                             closingQuote - openingQuote - 1);
      }
    }    
    options[option].push_back(value);
  }
  file.close();
  return true;
}

Configuration::operator bool() const {
  return !_error;
}

std::string Configuration::error() const {
  return errorMessage;
}

std::string Configuration::fileName() const {
  return _fileName;
}

std::string Configuration::getString(const std::string option) {
  if (options[option].size()) {
    return options[option][0];
  }
  return "";
}

std::vector <std::string> &Configuration::getStrings(const std::string option) {
  return options[option];
}

size_t Configuration::getNumber(const std::string option) {
  if (options[option].size()) {
    return strtoul(options[option][0].c_str(), NULL, 10);
  }
  return std::numeric_limits <size_t>::max();
}

#endif

==> geomWatch-1.3.0.tbz: geomWatch-1.3.0/configuration.hpp <==
/*
 * Copyright 2007-2011 Boris Kochergin. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef CONFIGURATION_HPP
#define CONFIGURATION_HPP

#include <fstream>
#include <limits>
#include <map>
#include <string>
#include <vector>

#include <errno.h>
#include <string.h>

class Configuration {
  public:
    Configuration(const std::string);
    Configuration();
    bool initialize(const std::string);
    operator bool() const;
    std::string error() const;
    std::string fileName() const;
    std::string getString(const std::string);
    std::vector <std::string> &getStrings(const std::string);
    size_t getNumber(const std::string);
  private:
    std::map <std::string, std::vector <std::string> > options;
    std::string _fileName;
    bool _error;
    std::string errorMessage;
};

Configuration::Configuration(const std::string fileName) {
  initialize(fileName);
}

Configuration::Configuration() {
  _error = true;
  errorMessage = "Configuration::Configuration(): class not initialized";
}

bool Configuration::initialize(const std::string fileName) {
  std::string line, option, value;
  size_t delimiter, openingQuote, closingQuote;
  std::ifstream file;
  _fileName = fileName;
  file.open(fileName.c_str());
  if (!file) {
    _error = true;
    errorMessage = "Configuration::initialize(): " + fileName + ": " +
                   strerror(errno);
    return false;
  }
  while (getline(file, line)) {
    delimiter = line.find('=');
    option = line.substr(0, delimiter);
    value = line.substr(delimiter + 1);
    openingQuote = value.find('"');
    if (openingQuote != std::string::npos) {
      closingQuote = value.find('"', openingQuote + 1);
      if (closingQuote != std::string::npos) {
        value = value.substr(openingQuote + 1,
                             closingQuote - openingQuote - 1);
      }
    }    
    options[option].push_back(value);
  }
  file.close();
  _error = false;
  errorMessage.clear();
  return true;
}

Configuration::operator bool() const {
  return !_error;
}

std::string Configuration::error() const {
  return errorMessage;
}

std::string Configuration::fileName() const {
  return _fileName;
}

std::string Configuration::getString(const std::string option) {
  if (options[option].size()) {
    return options[option][0];
  }
  return "";
}

std::vector <std::string> &Configuration::getStrings(const std::string option) {
  return options[option];
}

size_t Configuration::getNumber(const std::string option) {
  if (options[option].size()) {
    return strtoul(options[option][0].c_str(), NULL, 10);
  }
  return std::numeric_limits <size_t>::max();
}

#endif
